/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        24.10.2007
 Description:    Webserver
----------------------------------------------------------------------------*/

#include <avr/io.h>
#include <avr/interrupt.h>

#include "usart.h"
#include "enc28j60.h"
#include "stack.h"
#include "timer.h"
#include "httpd.h"
#include "cmd.h"

//----------------------------------------------------------------------------
//
ISR (INT2_vect)
{ 	
	while( (PINB &(1<<PB2)) == 0)
	{
		//Pointer auf Ethernet_Header
		struct Ethernet_Header *ethernet;
		ethernet = (struct Ethernet_Header *)&eth_buffer[ETHER_OFFSET];
		//Pointer auf IP_Header
		struct IP_Header *ip;
		ip = (struct IP_Header *)&eth_buffer[IP_OFFSET];
		//Pointer auf TCP_Header
		struct TCP_Header *tcp;
		tcp = (struct TCP_Header *)&eth_buffer[TCP_OFFSET];
		
		struct ICMP_Header *icmp;
		icmp = (struct ICMP_Header *)&eth_buffer[ICMP_OFFSET];
		
		unsigned int packet_lenght;
		  
		packet_lenght = enc28j60PacketReceive(MTU_SIZE,eth_buffer);
		/*Wenn ein Packet angekommen ist, ist packet_lenght =! 0*/
		packet_lenght = packet_lenght - 4;
		
		if(ETHERNET_ARP_DATAGRAMM)
		{
			//usart_write("ARP\r\n");
			//Erzeugt ein ARP Reply Packet
			arp_entry_add(eth_buffer); //Rechner schon Eingetragen?
			arp_reply(eth_buffer,&packet_lenght);
		}
		else
		{
			if(ETHERNET_IP_DATAGRAMM && MYIP)
			{
				arp_entry_add(eth_buffer); //Rechner schon Eingetragen?
				//Ist protokoll Byte = 1 dann ist es ein ICMP Packet
				if(IP_ICMP_PACKET)
				{
					//Erzeugt ein ICMP Reply Packet (PING Echo)
					icmp_reply(ip->IP_Srcaddr,icmp->ICMP_SeqNum);
				}
				else
				{
					if(IP_TCP_PACKET) tcp_socket_process(eth_buffer,&packet_lenght);
				}
			}
		}
	}
}

//----------------------------------------------------------------------------
//
int main(void)
{  
    myip[0] = 192;
    myip[1] = 168;
    myip[2] = 0;
    myip[3] = 99;
	
	mymac[0] = 0x00;
    mymac[1] = 0x20;
    mymac[2] = 0x18;
    mymac[3] = 0xb1;
    mymac[4] = 0x15;
    mymac[5] = 0x5f;
    
    unsigned long a;

#if defined (__AVR_ATmega32__)
	GICR |= (1<<INT2);
#endif

#if defined (__AVR_ATmega644__)
	EIMSK |= (1<<INT2);
#endif
	
    usart_init(9600); // setup the UART
	timer_init(); //Timer starten 
   
	usart_write("\n\rSystem Ready\n\r");	
    
 	/*ENC Initialisieren*/
    for(a=0;a<1000000;a++){asm("nop");};
	enc28j60Init();
	usart_write("NIC Init Ready\n\r");
    for(a=0;a<1000000;a++){asm("nop");};
	
	usart_write("My IP: %2i.%2i.%2i.%2i\r\n\r\n",myip[0],myip[1],myip[2],myip[3]);
	
	sei(); 
	
	enc28j60PhyWrite(0x14,0x0420); //0420 0AA0
 
	while(1)
	{
		if (usart_status.usart_ready)
		{
			if(cmd_exec(&usart_rx_buffer[0]))
			{
				usart_write("Ready\r\n\r\n");
			}
			else
			{
				usart_write("ERROR\r\n\r\n");
			}
			usart_status.usart_ready =0;
		}
    }
		
return(0);
}

