/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        24.10.2007
 Description:    
----------------------------------------------------------------------------*/

#ifndef _STACK_H
 #define _STACK_H

#include <string.h>
#include <avr/io.h>

#include "enc28j60.h"
#include "usart.h"
#include "httpd.h"

typedef struct
{
	unsigned int port;		// Port
	void(*fp)(unsigned char);  	// Zeiger auf auszufhrende Funktion
} PORT_ITEM;	

#define LBBL_ENDIAN_INT(x)	((x & 0x00FF)<<8)+((x & 0xFF00)>>8)
#define LBBL_ENDIAN_LONG(x) ((x & 0xFF000000)>>24)+((x & 0x00FF0000)>>8)+((x & 0x0000FF00)<<8)+((x & 0x000000FF)<<24)

unsigned char mymac[6];
unsigned char myip[4];

#define MTU_SIZE 1000
unsigned char eth_buffer[MTU_SIZE];


#define DEBUG usart_write

#define MAX_TCP_ENTRY 5
#define TCP_MAX_ENTRY_TIME 5

#define MAX_TCP_ERRORCOUNT 5
#define MAX_ARP_ENTRY 5
#define ARP_MAX_ENTRY_TIME 100 //100sec.

struct arp_table
{
	volatile unsigned char arp_t_mac[6];
	volatile unsigned long arp_t_ip;
	volatile unsigned int arp_t_time;
};

struct tcp_table
{
	unsigned long ip;
	unsigned int  src_port;
	unsigned int  dest_port;
	unsigned long ack_counter;
	unsigned long seq_counter;
	unsigned char status;
	unsigned int  app_status;
	unsigned char time;
	unsigned char error_count;
};

//----------------------------------------------------------------------------
//Prototypen
void new_eth_header (unsigned char *,unsigned long);

char arp_entry_search (unsigned long);
void arp_reply (unsigned char *, unsigned int *);
void arp_entry_add (unsigned char *);

void make_ip_header (unsigned char *,unsigned long);

void icmp_reply (unsigned long,unsigned int);
unsigned int checksum (unsigned char *,unsigned int,unsigned long);

void tcp_entry_add (unsigned char *);
void tcp_socket_process(unsigned char *,unsigned int *);
char tcp_entry_search (unsigned long ,unsigned int);
void tcp_Port_close (unsigned char);
void create_new_tcp_packet(unsigned int,unsigned char);

void find_and_start (unsigned char index);

#define ETHER_OFFSET			0x00
#define ARP_OFFSET				0x0E
#define IP_OFFSET				0x0E
#define ICMP_OFFSET				0x22
#define ICMP_DATA				0x2A
#define TCP_OFFSET				0x22

struct arp_table arp_entry[MAX_ARP_ENTRY];
struct tcp_table tcp_entry[MAX_TCP_ENTRY];

//IP Protocol Types
#define	PROT_ICMP				0x01	//zeigt an die Nutzlasten enthalten das ICMP Prot
#define	PROT_TCP				0x06	//zeigt an die Nutzlasten enthalten das TCP Prot.
#define	PROT_UDP				0x11	//zeigt an die Nutzlasten enthalten das UDP Prot.	

//Defines fr IF Abfrage
#define MYIP 			ip->IP_Destaddr==*((unsigned long*)&myip[0])
#define IP_UDP_PACKET 	ip->IP_Proto == PROT_UDP
#define IP_TCP_PACKET 	ip->IP_Proto == PROT_TCP
#define IP_ICMP_PACKET 	ip->IP_Proto == PROT_ICMP

//Defines fr if Abfrage
#define ETHERNET_IP_DATAGRAMM	ethernet->EnetPacketType == 0x0008
#define ETHERNET_ARP_DATAGRAMM 	ethernet->EnetPacketType == 0x0608

#define IP_VERS_LEN 20
#define TCP_HDR_LEN 20
#define ETH_HDR_LEN 14
#define TCP_DATA_START IP_VERS_LEN+TCP_HDR_LEN+ETH_HDR_LEN

#define MAX_WINDOWS_SIZE 0xFFFF

//TCP Flags
#define  FIN_FLAG			0x01	//Datenbertragung beendet (finished)
#define  SYN_FLAG			0x02	//Verbindungs aufbauen (synchronize)
#define  RST_FLAG			0x04	//Verbindung neu initialisieren (reset)
#define  PSH_FLAG			0x08	//Datenbergabe an Anwendung beschleunigen (push)
#define  ACK_FLAG			0x10	//Datenbertragung besttigen (acknowledged)

//----------------------------------------------------------------------------
//Aufbau eines Ethernetheader
//
//
//
//
struct Ethernet_Header	{
	unsigned char EnetPacketDest[6];	//MAC Zieladresse 6 Byte
	unsigned char EnetPacketSrc[6];	//MAC Quelladresse 6 Byte
	unsigned int EnetPacketType;		//Nutzlast 0x0800=IP Datagramm;0x0806 = ARP
};

//----------------------------------------------------------------------------
//Aufbau eines ARP Header
//	
//	2 BYTE Hardware Typ				|	2 BYTE Protokoll Typ	
//	1 BYTE Lnge Hardwareadresse	|	1 BYTE Lnge Protokolladresse
//	2 BYTE Operation
//	6 BYTE MAC Adresse Absender		|	4 BYTE IP Adresse Absender
//	6 BYTE MAC Adresse Empfnger	|	4 BYTE IP Adresse Empfnger	
struct ARP_Header	{
		unsigned int ARP_HWType;		//Hardware Typ enthlt den Code fr Ethernet oder andere Link Layer
		unsigned int ARP_PRType;		//Protokoll Typ enthlt den Code fr IP o. anderes bertragungsprotokoll
		unsigned char ARP_HWLen;		//Lnge der Hardwareadresse enthlt 6 fr 6 Byte lange MAC Adressen
		unsigned char ARP_PRLen;		//Lnge der Protocolladresse enthlt 4 fr 4 Byte lange IP Adressen
		unsigned int ARP_Op;			//Enthlt Code der signalisiert ob es sich um eine Anfrage o. Antwort handelt
		unsigned char ARP_SHAddr[6];	//Enthlt die MAC Adresse des Anfragenden  
		unsigned long ARP_SIPAddr;    //Enthlt die IP Adresse des Absenders
		unsigned char ARP_THAddr[6];	//MAC Adresse Ziel, ist in diesem Fall 6 * 00,da die Adresse erst noch herausgefunden werden soll (ARP Request)
		unsigned long ARP_TIPAddr;    //IP Adresse enthlt die IP Adresse zu der die Kommunikation aufgebaut werden soll 
};

//----------------------------------------------------------------------------
//Aufbau eines IP Datagramms (B=BIT)
//	
//4B Version	|4B Headergr.	|8B Tos	|16B Gesamtlnge in Bytes	
//16B Identifikation			|3B Schalter	|13B Fragmentierungsposition
//8B Time to Live	|8B Protokoll	|16B Header Prfsumme 
//32B IP Quelladresse
//32B IB Zieladresse
struct IP_Header	{
	unsigned char	IP_Vers_Len;	//4 BIT Die Versionsnummer von IP, 
									//meistens also 4 + 4Bit Headergre 	
	unsigned char	IP_Tos;			//Type of Service
	unsigned int	IP_Pktlen;		//16 Bit Komplette Lng des IP Datagrams in Bytes
	unsigned int	IP_Id;			//ID des Packet fr Fragmentierung oder 
									//Reassemblierung
	unsigned int	IP_Frag_Offset;	//wird benutzt um ein fragmentiertes 
									//IP Packet wieder korrekt zusammenzusetzen
	unsigned char	IP_ttl;			//8 Bit Time to Live die lebenszeit eines Paket
	unsigned char	IP_Proto;		//Zeigt das hherschichtige Protokoll an 
									//(TCP, UDP, ICMP)
	unsigned int	IP_Hdr_Cksum;	//Checksumme des IP Headers
	unsigned long	IP_Srcaddr;		//32 Bit IP Quelladresse
	unsigned long	IP_Destaddr;	//32 Bit IP Zieladresse
};

//----------------------------------------------------------------------------
//Aufbau einer ICMP Nachricht
//	
//8 BIT Typ	|8 BIT Code	|16 BIT Prfsumme	
//Je nach Typ			|Nachricht unterschiedlich
//Testdaten
//
//8 BIT Typ = 0 Echo Reply oder 8 Echo request	
struct ICMP_Header {
		unsigned char     	ICMP_Type;		//8 bit typ identifiziert Aufgabe der Nachricht 
											//0 = Echo Reply 8 = Echo Request
		unsigned char     	ICMP_Code;		//8 Bit Code enthlt Detailangaben zu bestimmten Typen
		unsigned int     	ICMP_Cksum;		//16 Bit Prfsumme enthlt die CRC Prfsumme
		unsigned int     	ICMP_Id;		//2 byte Identifizierung
		unsigned int     	ICMP_SeqNum;	//Sequenznummer
};

//----------------------------------------------------------------------------
//TCP Header Layout
//
//
struct TCP_Header {
	unsigned int 	TCP_SrcPort;	//der Quellport fr die Verbindung (Socket)
	unsigned int 	TCP_DestPort;	//der Zielport fr die Verbindung (Socket)
	unsigned long	TCP_Seqnum;	//numerierter Bytestrom
	unsigned long	TCP_Acknum;	//numerierter Bytestrom (Gegenrichtung)
	unsigned char  TCP_Hdrlen;
	unsigned char 	TCP_HdrFlags;//4Bit Lnge des Headers in 32 Bit schritten rest Flags
	unsigned int	TCP_Window;	//Fenstergre max. 64K Byte
	unsigned int	TCP_Chksum;	//Enthlt eine Prfsumme ber Header und Datenbereich
	unsigned int	TCP_UrgentPtr;	//16 Bit fr dringende bertragung
};


#endif // #define _STACK_H
